package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.MD5Util;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.ChannelDTO;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.dto.UserDTO;
import com.zanxiang.manage.domain.dto.UserExtDTO;
import com.zanxiang.manage.domain.params.UserListParam;
import com.zanxiang.manage.domain.params.UserUpdateParam;
import com.zanxiang.manage.domain.vo.GameUserVO;
import com.zanxiang.manage.domain.vo.UserListVO;
import com.zanxiang.manage.domain.vo.UserVO;
import com.zanxiang.manage.service.*;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserExt;
import com.zanxiang.mybatis.mapper.UserMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户玩家
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private UserExtService userExtService;

    @Autowired
    private GameService gameService;

    @Autowired
    private ChannelService channelService;

    @Autowired
    private GameUserService gameUserService;

    /**
     * 玩家列表查询
     *
     * @param param : 查询参数
     * @return : 返回单页玩家数据
     */
    @Override
    public IPage<UserListVO> list(UserListParam param) {
        //游戏条件处理
        Map<Long, GameDTO> gameMap = gameService.gameCondition(param.getCpId(), param.getGameId(), param.getGameCategoryId());
        //根据条件, 匹配不到游戏
        if (gameMap != null && gameMap.isEmpty()) {
            return new Page<>();
        }
        //玩家拓展信息条件
        Map<Long, UserExtDTO> userExtMap = userExtService.getUserExtList(param.getRealName(), param.getRegIp(), param.getIsAuth());
        if (userExtMap != null && userExtMap.isEmpty()) {
            return new Page<>();
        }
        //判断指定的玩家id是否在用户列表中
        if (param.getUserId() != null && userExtMap != null && !userExtMap.keySet().contains(param.getUserId())) {
            return new Page<>();
        }
        //执行查询
        return page(param.toPage(), new QueryWrapper<User>().lambda()
                .eq(param.getUserId() != null, User::getId, param.getUserId())
                .like(Strings.isNotBlank(param.getUserName()), User::getUsername, param.getUserName())
                .like(Strings.isNotBlank(param.getNickname()), User::getNickname, param.getNickname())
                .eq(Strings.isNotBlank(param.getMobile()), User::getMobile, param.getMobile())
                .in(userExtMap != null, User::getId, userExtMap != null ? userExtMap.keySet() : null)
                .isNotNull(Objects.equals(param.getIsBindMobile(), Boolean.TRUE), User::getMobile)
                .isNull(Objects.equals(param.getIsBindMobile(), Boolean.FALSE), User::getMobile)
                .in(gameMap != null, User::getGameId, gameMap != null ? gameMap.keySet() : null)
                .ge(param.getBeginDate() != null, User::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, User::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .ge(param.getRechargeBeginDate() != null, User::getLastRechargeTime, param.getRechargeBeginDate() == null ? null : LocalDateTime.of(param.getRechargeBeginDate(), LocalTime.MIN))
                .le(param.getRechargeEndDate() != null, User::getLastRechargeTime, param.getRechargeEndDate() == null ? null : LocalDateTime.of(param.getRechargeEndDate(), LocalTime.MAX))
                .gt(Objects.equals(param.getIsRecharge(), Boolean.TRUE), User::getRechargeCount, 0)
                .eq(Objects.equals(param.getIsRecharge(), Boolean.FALSE), User::getRechargeCount, 0)
                .eq(param.getVipLevel() != null && param.getVipLevel() <= 10, User::getVipMax, param.getVipLevel())
                .gt(param.getVipLevel() != null && param.getVipLevel() > 10, User::getVipMax, param.getVipLevel())
                .eq(param.getStatus() != null, User::getStatus, param.getStatus())
                .eq(param.getChannelId() != null, User::getAgentId, param.getChannelId())
                // todo : 归因推广账号
                // todo : 归因投放人员
                .eq(param.getCustomerId() != null, User::getCustomerId, param.getCustomerId())
                // todo : 归因广告
                .eq(param.getIsGs() != null, User::getIsGs, param.getIsGs())
                // todo : 是否切量
                .orderByDesc(User::getCreateTime)
        ).convert(u -> this.toVo(u, userExtMap, gameMap));
    }

    /**
     * 游戏玩家信息转换对象
     *
     * @param user       : 用户信息
     * @param userExtMap : 用户拓展信息
     * @param gameMap    : 游戏列表
     * @return : 返回用户信息
     */
    private UserListVO toVo(User user, Map<Long, UserExtDTO> userExtMap, Map<Long, GameDTO> gameMap) {
        UserListVO userListVO = BeanUtils.copy(user, UserListVO.class);
        if (Objects.isNull(userListVO)) {
            return null;
        }
        UserExtDTO userExtDTO = StringUtils.isEmpty(userExtMap) ? null : userExtMap.get(userListVO.getId());
        if (userExtDTO == null) {
            userExtDTO = userExtService.getById(userListVO.getId());
        }
        if (userExtDTO != null) {
            userListVO.setRealName(userExtDTO.getRealName());
            userListVO.setRegIp(userExtDTO.getRegIp());
            userListVO.setIdCard(userExtDTO.getIdCard());
        }
        GameDTO gameDTO = StringUtils.isEmpty(gameMap) ? null : gameMap.get(userListVO.getGameId());
        if (gameDTO == null) {
            gameDTO = gameService.getById(userListVO.getGameId());
        }
        if (gameDTO != null) {
            userListVO.setGameCategoryId(gameDTO.getCategory());
            userListVO.setCpId(gameDTO.getCpId());
        }
        userListVO.setChannelId(user.getAgentId());
        return userListVO;
    }

    /**
     * 用户信息更新
     *
     * @param param : 用户更新参数
     * @return : 返回更新结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean userUpdate(UserUpdateParam param) {
        //用户信息更新
        if (Strings.isNotBlank(param.getMobile()) || Strings.isNotBlank(param.getAliPay()) || Strings.isNotBlank(param.getPassword())) {
            super.update(new LambdaUpdateWrapper<User>()
                    .eq(User::getId, param.getUserId())
                    .eq(User::getUsername, param.getUsername())
                    .set(Strings.isNotBlank(param.getMobile()), User::getMobile, param.getMobile())
                    .set(Strings.isNotBlank(param.getAliPay()), User::getAliPay, param.getAliPay())
                    .set(Strings.isNotBlank(param.getPassword()), User::getPassword, this.cmfPassword(param.getPassword()))
                    .set(User::getUpdateTime, LocalDateTime.now()));
        }
        //用户拓展信息更新
        if (Strings.isNotBlank(param.getRealName()) || Strings.isNotBlank(param.getIdCard())) {
            userExtService.update(new LambdaUpdateWrapper<UserExt>()
                    .eq(UserExt::getUserId, param.getUserId())
                    .set(Strings.isNotBlank(param.getRealName()), UserExt::getRealName, param.getRealName())
                    .set(Strings.isNotBlank(param.getIdCard()), UserExt::getIdCard, param.getIdCard())
                    .set(UserExt::getUpdateTime, LocalDateTime.now()));
        }
        return Boolean.TRUE;
    }

    /**
     * 密码cfm加密
     *
     * @param password : 密码
     * @return : 返回加密完的密码
     */
    private String cmfPassword(String password) {
        String passwordSalt = password + "ZX_PASSWORD_SALT";
        String charsetName = Constants.UTF8;
        return "###" + MD5Util.MD5Encode(MD5Util.MD5Encode(passwordSalt, charsetName), charsetName);
    }

    /**
     * 查询玩家详细信息
     *
     * @param userId : 用户id
     * @return : 返回用户信息
     */
    @Override
    public UserVO getUserInfo(Long userId) {
        //查询用户信息
        User user = super.getById(userId);
        UserVO userVO = BeanUtils.copy(user, UserVO.class);
        if (userVO == null) {
            throw new BaseException("用户信息不存在!");
        }
        //查询用户拓展信息
        UserExtDTO userExtDTO = userExtService.getById(userId);
        if (userExtDTO == null) {
            throw new BaseException("用户拓展信息不存在!");
        }
        //拓展信息补充
        userVO.setRealName(userExtDTO.getRealName());
        userVO.setBirthday(userExtDTO.getBirthday());
        userVO.setSex(userExtDTO.getSex());
        userVO.setIdCard(userExtDTO.getIdCard());
        userVO.setRegMobile(userExtDTO.getRegMobile());
        userVO.setRegEmail(userExtDTO.getRegEmail());
        userVO.setRegFrom(userExtDTO.getRegFrom());
        userVO.setRegIp(userExtDTO.getRegIp());
        //注册游戏
        GameDTO gameDTO = gameService.getById(userExtDTO.getRegGameId());
        if (gameDTO != null) {
            userVO.setRegGame(gameDTO.getName());
        }
        //渠道
        ChannelDTO channelDTO = channelService.getById(user.getAgentId());
        if (channelDTO != null) {
            userVO.setChannelName(channelDTO.getChannelName());
        }
        //vip等级
        GameUserVO maxVipGameUser = gameUserService.getMaxVipGameUser(userId);
        if (maxVipGameUser != null) {
            userVO.setVipMax(maxVipGameUser.getRoleVipMax());
        }
        //地址
        userVO.setAddress("贝达梦工厂E座二楼趣程科技");
        return userVO;
    }


    /**
     * 玩家相关条件查询用户信息
     *
     * @param userId      : 用户id
     * @param agentIdList : 渠道id列表
     * @param userName    : 用户名称
     * @param nickname    : 用户昵称
     * @param regIp       : 用户注册ip
     * @param adId        : 广告id
     * @return : 返回用户信息map
     */
    @Override
    public Map<Long, UserDTO> userCondition(Long userId, List<Long> agentIdList, String userName, String nickname,
                                            String regIp, String adId) {
        //用户信息条件处理
        if (userId == null && CollectionUtils.isEmpty(agentIdList) && Strings.isBlank(userName)
                && Strings.isBlank(nickname) && Strings.isBlank(regIp) && Strings.isBlank(adId)) {
            return null;
        }
        //查询
        List<User> userList = super.list(new LambdaQueryWrapper<User>()
                .eq(userId != null, User::getId, userId)
                .eq(Strings.isNotBlank(adId), User::getAdId, adId)
                .in(CollectionUtils.isNotEmpty(agentIdList), User::getAgentId, agentIdList)
                .eq(Strings.isNotBlank(regIp), User::getIp, regIp)
                .like(Strings.isNotBlank(userName), User::getUsername, userName)
                .like(Strings.isNotBlank(nickname), User::getNickname, nickname)
                .select(User::getId, User::getUsername, User::getNickname, User::getIp, User::getCreateTime, User::getAgentId, User::getAdId));
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        List<UserDTO> list = BeanUtils.copyList(userList, UserDTO.class);
        //构造返回
        return list.stream().collect(Collectors.toMap(UserDTO::getId, Function.identity()));
    }

    /**
     * 根据用户id查询信息
     *
     * @param userId : 返回用户信息
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getById(Long userId) {
        return BeanUtils.copy(super.getById(userId), UserDTO.class);
    }

}
